@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -27,14 +27,11 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "precompiled.hpp"
 #include <stdlib.h>
 #include <string.h>
 #include <limits>
-
-#include "platform.hpp"
-#if defined ZMQ_HAVE_WINDOWS
-#include "windows.hpp"
-#endif
+#include <limits.h>
 
 #include "decoder.hpp"
 #include "v1_decoder.hpp"
@@ -43,64 +40,63 @@
 #include "err.hpp"
 
 zmq::v1_decoder_t::v1_decoder_t (size_t bufsize_, int64_t maxmsgsize_) :
-    decoder_base_t <v1_decoder_t> (bufsize_),
-    maxmsgsize (maxmsgsize_)
+    decoder_base_t<v1_decoder_t> (bufsize_),
+    _max_msg_size (maxmsgsize_)
 {
-    int rc = in_progress.init ();
+    int rc = _in_progress.init ();
     errno_assert (rc == 0);
 
     //  At the beginning, read one byte and go to one_byte_size_ready state.
-    next_step (tmpbuf, 1, &v1_decoder_t::one_byte_size_ready);
+    next_step (_tmpbuf, 1, &v1_decoder_t::one_byte_size_ready);
 }
 
 zmq::v1_decoder_t::~v1_decoder_t ()
 {
-    int rc = in_progress.close ();
+    const int rc = _in_progress.close ();
     errno_assert (rc == 0);
 }
 
-int zmq::v1_decoder_t::one_byte_size_ready ()
+int zmq::v1_decoder_t::one_byte_size_ready (unsigned char const *)
 {
-    //  First byte of size is read. If it is 0xff read 8-byte size.
+    //  First byte of size is read. If it is UCHAR_MAX (0xff) read 8-byte size.
     //  Otherwise allocate the buffer for message data and read the
     //  message data into it.
-    if (*tmpbuf == 0xff)
-        next_step (tmpbuf, 8, &v1_decoder_t::eight_byte_size_ready);
+    if (*_tmpbuf == UCHAR_MAX)
+        next_step (_tmpbuf, 8, &v1_decoder_t::eight_byte_size_ready);
     else {
-
         //  There has to be at least one byte (the flags) in the message).
-        if (!*tmpbuf) {
+        if (!*_tmpbuf) {
             errno = EPROTO;
             return -1;
         }
 
-        if (maxmsgsize >= 0 && (int64_t) (*tmpbuf - 1) > maxmsgsize) {
+        if (_max_msg_size >= 0
+            && static_cast<int64_t> (*_tmpbuf - 1) > _max_msg_size) {
             errno = EMSGSIZE;
             return -1;
         }
 
-        //  in_progress is initialised at this point so in theory we should
-        //  close it before calling zmq_msg_init_size, however, it's a 0-byte
-        //  message and thus we can treat it as uninitialised...
-        int rc = in_progress.init_size (*tmpbuf - 1);
+        int rc = _in_progress.close ();
+        assert (rc == 0);
+        rc = _in_progress.init_size (*_tmpbuf - 1);
         if (rc != 0) {
             errno_assert (errno == ENOMEM);
-            rc = in_progress.init ();
+            rc = _in_progress.init ();
             errno_assert (rc == 0);
             errno = ENOMEM;
             return -1;
         }
 
-        next_step (tmpbuf, 1, &v1_decoder_t::flags_ready);
+        next_step (_tmpbuf, 1, &v1_decoder_t::flags_ready);
     }
     return 0;
 }
 
-int zmq::v1_decoder_t::eight_byte_size_ready ()
+int zmq::v1_decoder_t::eight_byte_size_ready (unsigned char const *)
 {
     //  8-byte payload length is read. Allocate the buffer
     //  for message body and read the message data into it.
-    const uint64_t payload_length = get_uint64 (tmpbuf);
+    const uint64_t payload_length = get_uint64 (_tmpbuf);
 
     //  There has to be at least one byte (the flags) in the message).
     if (payload_length == 0) {
@@ -109,50 +105,52 @@ int zmq::v1_decoder_t::eight_byte_size_ready ()
     }
 
     //  Message size must not exceed the maximum allowed size.
-    if (maxmsgsize >= 0 && payload_length - 1 > (uint64_t) maxmsgsize) {
+    if (_max_msg_size >= 0
+        && payload_length - 1 > static_cast<uint64_t> (_max_msg_size)) {
         errno = EMSGSIZE;
         return -1;
     }
 
+#ifndef __aarch64__
     //  Message size must fit within range of size_t data type.
-    if (payload_length - 1 > std::numeric_limits <size_t>::max ()) {
+    if (payload_length - 1 > std::numeric_limits<size_t>::max ()) {
         errno = EMSGSIZE;
         return -1;
     }
+#endif
 
-    const size_t msg_size = static_cast <size_t> (payload_length - 1);
+    const size_t msg_size = static_cast<size_t> (payload_length - 1);
 
-    //  in_progress is initialised at this point so in theory we should
-    //  close it before calling init_size, however, it's a 0-byte
-    //  message and thus we can treat it as uninitialised...
-    int rc = in_progress.init_size (msg_size);
+    int rc = _in_progress.close ();
+    assert (rc == 0);
+    rc = _in_progress.init_size (msg_size);
     if (rc != 0) {
         errno_assert (errno == ENOMEM);
-        rc = in_progress.init ();
+        rc = _in_progress.init ();
         errno_assert (rc == 0);
         errno = ENOMEM;
         return -1;
     }
 
-    next_step (tmpbuf, 1, &v1_decoder_t::flags_ready);
+    next_step (_tmpbuf, 1, &v1_decoder_t::flags_ready);
     return 0;
 }
 
-int zmq::v1_decoder_t::flags_ready ()
+int zmq::v1_decoder_t::flags_ready (unsigned char const *)
 {
     //  Store the flags from the wire into the message structure.
-    in_progress.set_flags (tmpbuf [0] & msg_t::more);
+    _in_progress.set_flags (_tmpbuf[0] & msg_t::more);
 
-    next_step (in_progress.data (), in_progress.size (),
-        &v1_decoder_t::message_ready);
+    next_step (_in_progress.data (), _in_progress.size (),
+               &v1_decoder_t::message_ready);
 
     return 0;
 }
 
-int zmq::v1_decoder_t::message_ready ()
+int zmq::v1_decoder_t::message_ready (unsigned char const *)
 {
     //  Message is completely read. Push it further and start reading
     //  new message. (in_progress is a 0-byte message after this point.)
-    next_step (tmpbuf, 1, &v1_decoder_t::one_byte_size_ready);
+    next_step (_tmpbuf, 1, &v1_decoder_t::one_byte_size_ready);
     return 1;
 }
